"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionController = void 0;
const bcrypt_1 = require("bcrypt");
const jsonwebtoken_1 = require("jsonwebtoken");
const user_record_1 = require("../records/user-record");
const handleError_1 = require("../utils/handleError");
const config_1 = require("../config/config");
const types_1 = require("../types");
class SessionController {
    static create(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const { email, password } = req.body;
            if (!email || !password)
                throw new handleError_1.CustomError('Email and password are required.', 400);
            const user = yield user_record_1.UserRecord.getByEmail(email);
            if (!user)
                throw new handleError_1.CustomError('Wrong email or password', 401);
            const match = yield (0, bcrypt_1.compare)(password, user.password);
            if (!match)
                throw new handleError_1.CustomError('Wrong email or password', 401);
            if (user.accountStatus === types_1.AccountStatus.PENDING)
                throw new handleError_1.CustomError('Email is not verified yet', 401);
            const accessToken = (0, jsonwebtoken_1.sign)({ id: user.id }, config_1.config.ACCESS_TOKEN_SECRET, { expiresIn: '15min' });
            const refreshToken = (0, jsonwebtoken_1.sign)({ id: user.id }, config_1.config.REFRESH_TOKEN_SECRET, { expiresIn: '7d' });
            user.refreshToken = refreshToken;
            yield user.update();
            res.cookie('__refresh', refreshToken, {
                httpOnly: true,
                secure: true,
                sameSite: 'none',
                maxAge: 1000 * 60 * 60 * 24 * 7,
            });
            res.json({ id: user.id, accessToken });
        });
    }
    static refresh(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const refreshToken = req.cookies['__refresh'];
                if (!refreshToken)
                    return res.sendStatus(401);
                const { id } = (0, jsonwebtoken_1.verify)(refreshToken, config_1.config.REFRESH_TOKEN_SECRET);
                const user = yield user_record_1.UserRecord.getById(id);
                if (!user)
                    return res.sendStatus(403);
                const accessToken = (0, jsonwebtoken_1.sign)({ id }, config_1.config.ACCESS_TOKEN_SECRET, { expiresIn: '15min' });
                user.refreshToken = (0, jsonwebtoken_1.sign)({ id: user.id }, config_1.config.REFRESH_TOKEN_SECRET, { expiresIn: '7d' });
                yield user.update();
                res.json(accessToken);
            }
            catch (e) {
                if (e.message === 'invalid token') {
                    res.sendStatus(403);
                }
                else {
                    res.sendStatus(500);
                }
            }
        });
    }
    static logout(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const refreshToken = req.cookies['__refresh'];
            if (!refreshToken)
                return res.sendStatus(200);
            const { id } = (0, jsonwebtoken_1.verify)(refreshToken, config_1.config.REFRESH_TOKEN_SECRET);
            const user = yield user_record_1.UserRecord.getById(id);
            if (!user)
                return res.sendStatus(200);
            user.refreshToken = null;
            yield user.update();
            res.clearCookie('__refresh', {
                httpOnly: true,
                secure: true,
                sameSite: 'none',
                maxAge: 1000 * 60 * 60 * 24 * 7,
            });
            res.sendStatus(200);
        });
    }
}
exports.SessionController = SessionController;
//# sourceMappingURL=session-controller.js.map